
#ifndef CALC_GATE_H_included
#define CALC_GATE_H_included

#define  KINETIC_DEFAULT_EPS 1.0e-6
#define  DT_MIN 1.0e-14   // smallest time step

//*************************************************************************************

class KineticObj : public VarList
{

 public:

  VectorObj      *var, *varOld[2];
  VectorObj      *ident;
  int            var_num;
  int            ident_num;
  char           **var_id;
  char           **ident_id;
  ExpressionObj  **formulas;

  TableArray     *tables;
  PeakTrackArray *maxima;
  InterpolArray  *locations;
  
  double Time, TimeOld[2];

  KineticObj(int n, int m) : var_num(n), ident_num(m), Time(0.0) { initialize(); }
  KineticObj(class SimulationObj *);
  void initialize();
  void Evaluate();

  VectorObj derivative();                      // returns dt satisfying accuracy eps
  double    RungeKuttaAdaptive(double T, double eps, class PlotArray *plots = 0, class RunStatusString *rss=0,
                               double total = 0.0, double dt = 0.0, long max_steps = 40000);   

  char      eq_flag;

  ~KineticObj();

  void saveState(int level = 0)    { *varOld[level] = *var; TimeOld[level] = Time; 
                     tables->saveState(level); maxima->saveState(level); locations->saveState(); }

  void recoverState(int level = 0) { *var = *varOld[level]; Time = TimeOld[level]; 
                     tables->recoverState(level); maxima->recoverState(level); locations->recoverState(); }

  double *ResolveID(const char *, double **t=0);
  const char *ResolvePtr(double *ptr);
};


//*************************************************************************************


#endif
